package com.alentoma.selectabletext;

import android.graphics.Rect;
import android.text.Editable;
import android.text.Layout;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.TextWatcher;
import android.util.Log;
import android.view.ActionMode;
import android.view.ActionMode.Callback;
import android.view.Menu;
import android.view.MenuItem;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.facebook.react.views.text.ReactTextView;
import com.facebook.react.views.text.ReactTextViewManager;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class RNSelectableTextManager extends CustomReactTextViewManager {
    public static final String REACT_CLASS = "RNSelectableText";
    private ReadableArray mHighlights = Arguments.createArray();

    @NonNull
    @Override
    public String getName() {
        return REACT_CLASS;
    }

    @ReactProp(name = "highlights")
    public void setHighlights(ReactTextView textView, ReadableArray highlights) {
        Log.d("Received Highlights", highlights.toString());

        mHighlights = highlights;

        // TODO: find a better way to register selection listener
        registerSelectionListener(textView);
        registerOnTextChangeListener(textView);
    }

    public void calculateHighlightRects(final ReactTextView textView) {
        WritableArray rects = Arguments.createArray();

        for (int i = 0; i < mHighlights.size(); i++) {
            ReadableMap highlight = mHighlights.getMap(i);
            int start = highlight.getInt("start");
            int end = highlight.getInt("end");

            WritableMap rectMap = Utils.getTextSelectionRect(textView, start, end);
            if (rectMap != null) {
                Log.d("Rect", rectMap.toString());

                WritableMap itemMap = Arguments.createMap();

                itemMap.putMap("rect", rectMap);
                itemMap.putString("id", highlight.getString("id"));

                rects.pushMap(itemMap);
            }
        }

        // dispatch event
        onHighlightRectsCalculatedEvent(textView, rects);
    }

    public void registerOnTextChangeListener(final ReactTextView view) {
        view.addTextChangedListener(new TextWatcher() {
            @Override
            public void afterTextChanged(Editable s) {
                calculateHighlightRects(view);
            }

            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                // do nothing
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                // do nothing
            }
        });
    }


    public void registerSelectionListener(final ReactTextView view) {
        view.setCustomSelectionActionModeCallback(new Callback() {
            @Override
            public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
                menu.clear();
                return true;
            }

            @Override
            public boolean onCreateActionMode(ActionMode mode, Menu menu) {
                return true;
            }

            @Override
            public void onDestroyActionMode(ActionMode mode) {
            }

            @Override
            public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
                mode.finish();
                return true;
            }

        });
    }

    /*
        interface SelectionChangeEvent {
            selection: {
                start: number;
                end: number;
                rect: {
                    x: number;
                    y: number;
                    width: number;
                    height: number;
                };
            };
        }
    */
    public void onTextSelectionChangeEvent(ReactTextView view, int start, int end, ReadableMap rect) {
        WritableMap selection = Arguments.createMap();
        selection.putInt("start", start);
        selection.putInt("end", end);
        selection.putMap("rect", rect);

        WritableMap event = Arguments.createMap();
        event.putMap("selection", selection);

        // Dispatch
        ReactContext reactContext = (ReactContext) view.getContext();
        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(view.getId(), "topOnTextSelectionChange", event);
    }

    /*
        interface HighlightRectsCalculatedNativeEvent {
            nativeEvent: {
                rects: Array<{
                    id: string;
                    rect: {
                        x: number;
                        y: number;
                        width: number;
                        height: number;
                    };
                }>;
            };
        }
     */
    public void onHighlightRectsCalculatedEvent(ReactTextView view, ReadableArray rects) {
        WritableMap event = Arguments.createMap();
        event.putArray("rects", rects);

        // Dispatch
        ReactContext reactContext = (ReactContext) view.getContext();
        reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(view.getId(), "topOnHighlightRectsCalculated", event);
    }

    @Override
    public Map<String, Object> getExportedCustomDirectEventTypeConstants() {
        MapBuilder.Builder<String, Object> builder = MapBuilder.builder();

        builder.put("topOnTextSelectionChange", MapBuilder.of("registrationName", "onTextSelectionChange"));
        builder.put("topOnHighlightRectsCalculated", MapBuilder.of("registrationName", "onHighlightRectsCalculated"));

        return builder.build();
    }
}
